import 'package:anagog_flutter_platform_interface/anagog_flutter_platform_interface.dart';
import 'package:anagog_flutter_platform_interface/src/model/micromoment.dart';
import 'package:flutter/services.dart';

class AnagogFlutterMethodChannel extends AnagogFlutterPlatform {
  final _methodChannel =
      const MethodChannel('com.anagog.flutter/method_channel');
  final _eventChannel =
      const EventChannel('com.anagog.flutter/method_channel/events');

  @override
  Future simulateCampaign(String? campaignIdentifier) async =>
      _methodChannel.invokeMethod('simulateCampaign',
          <String, dynamic>{CAMPAIGN_IDENTIFIER: campaignIdentifier});

  @override
  Future syncCampaigns() async => _methodChannel.invokeMethod('syncCampaigns');

  @override
  Future<Map<dynamic, dynamic>?> getUserDefinedStat() async =>
      _methodChannel.invokeMapMethod("getUserDefinedStat");

  @override
  Future<dynamic> setUserDefinedString(Map<String, dynamic> arguments) async =>
      _methodChannel.invokeMethod("setUserDefinedString", arguments);

  @override
  Future<String?> getUserDefinedString(String name) async =>
      _methodChannel.invokeMethod("getUserDefinedString", name);

  @override
  Future<dynamic> setUserDefinedInteger(Map<String, dynamic> arguments) async =>
      _methodChannel.invokeMethod("setUserDefinedInteger", arguments);

  @override
  Future<int?> getUserDefinedInteger(String name) async =>
      _methodChannel.invokeMethod("getUserDefinedInteger", name);

  @override
  Future<dynamic> setUserDefinedDecimal(Map<String, dynamic> arguments) async =>
      _methodChannel.invokeMethod("setUserDefinedDecimal", arguments);

  @override
  Future<double?> getUserDefinedDecimal(String name) async =>
      _methodChannel.invokeMethod("getUserDefinedDecimal", name);

  @override
  Future fireMicromoments(Micromoment micromoment) async =>
      _methodChannel.invokeMethod("fireMicromoments", micromoment.toJson());

  @override
  Future share() async => _methodChannel.invokeMethod("share");

  @override
  Stream obtainEventStream() => _eventChannel.receiveBroadcastStream();

  @override
  Future getVersion() async => _methodChannel.invokeMethod("getVersion");

  @override
  Future requestMicrosegmentsSnapshotReport() async =>
      _methodChannel.invokeMethod("requestMicrosegmentsSnapshotReport");

  @override
  Future forceScheduleReports() async =>
      _methodChannel.invokeMethod("forceScheduleReports");

  @override
  Future setUserInteractionEvent(
      String campaignIdentifier, AnagogActivityType activityType) async {
    Map<String, String> params = {};
    params[CAMPAIGN_IDENTIFIER] = campaignIdentifier;
    params[ACTIVITY_TYPE] = activityType.name;
    return _methodChannel.invokeMethod("setUserInteractionEvent", params);
  }
}
